@@ -7,10 +7,10 @@ use warnings;
 use vars qw($VERSION @EXPORT_OK %EXPORT_TAGS $DEBUG);
 
 $DEBUG = 0;
-$VERSION = '0.30';
+$VERSION = '0.31';
 
 use base qw(Class::Accessor);
-XML::Sig->mk_accessors(qw(canonicalizer key));
+XML::Sig->mk_accessors(qw(key));
 
 # We are exporting functions
 use base qw/Exporter/;
@@ -52,7 +52,6 @@ sub new {
     }
     bless $self, $class;
     $self->{ 'x509' } = exists $params->{ x509 } ? 1 : 0;
-    $self->{ 'exclusive' } = exists $params->{ exclusive } ? $params->{ exclusive } : 2;
     if ( exists $params->{ 'key' } ) {
         $self->_load_key( $params->{ 'key' } );
     }
@@ -123,17 +122,24 @@ sub sign {
     # Canonicalize the SignedInfo to http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments
     # TODO Change the Canonicalization method in the xml fragment from _signedinfo_xml
 
+    my ($signature_node) = $self->{ parser }->findnodes(
+        '//dsig:Signature');
     my ($signed_info_node) = $self->{ parser }->findnodes(
         '//dsig:Signature/dsig:SignedInfo');
-    my $signed_info_canon = $signed_info_node->toStringC14N(0);
+
+    # At this point the SignedInfo includes the information
+    # to allow us to use the _canonicalize_xml with the $signature_node
+    my $signed_info_canon = $self->_canonicalize_xml($signed_info_node, $signature_node);
 
     # Calculate the signature of the Canonical Form of SignedInfo
     my $signature;
     if ($self->{key_type} eq 'dsa') {
         print ("    Signing SignedInfo using DSA key type\n") if $DEBUG;
         # DSA only permits the signing of 20 bytes or less, hence the sha1
-        my $bin_signature = $self->{key_obj}->sign( sha1($signed_info_canon) );
-        $signature        = encode_base64( $bin_signature, "\n" );
+        my $bin_signature = $self->{key_obj}->do_sign( sha1($signed_info_canon) );
+        my $r = $bin_signature->get_r;
+        my $s = $bin_signature->get_s;
+        $signature        = encode_base64( $r . $s, "\n" );
     } else {
         print ("    Signing SignedInfo using RSA key type\n") if $DEBUG;
         my $bin_signature = $self->{key_obj}->sign( $signed_info_canon );
@@ -303,13 +309,6 @@ sub signer_cert {
     return $self->{signer_cert};
 }
 
-#TODO Remove Unused
-sub _get_num_signatures {
-    my $self = shift;
-    my $num = $self->{parser}->find('//dsig:Signature');
-    print "\n\nNumber of Signatures: $num\n\n";
-}
-
 sub _get_xml_to_sign {
     my $self = shift;
     my $id = $self->{parser}->findvalue('//@ID');
@@ -356,13 +355,13 @@ sub _transform {
            $xml = $xml->toStringC14N();
         }
         elsif ($alg eq TRANSFORM_C14N_COMMENTS) {
-            $xml = $xml->toStringC14N(1)
+            $xml = $xml->toStringC14N(1);
         }
         elsif ($alg eq TRANSFORM_EXC_C14N) {
             $xml = $xml->toStringEC14N();
         }
         elsif ($alg eq TRANSFORM_EXC_C14N_COMMENTS) {
-            $xml = $xml->toStringEC14N(1)
+            $xml = $xml->toStringEC14N(1);
         }
         else {
             die "Unsupported transform: $alg";
@@ -371,13 +370,6 @@ sub _transform {
     return $xml;
 }
 
-sub _find_prefixlist {
-    my $self = shift;
-    my ($node) = @_;
-    my $prefixlist = $self->{parser}->findvalue('ec:InclusiveNamespaces/@PrefixList', $node);
-    return $prefixlist;
-}
-
 sub _verify_rsa {
     my $self = shift;
     my ($context,$canonical,$sig) = @_;
@@ -469,6 +461,7 @@ sub _verify_dsa {
 
     eval {
         require Crypt::OpenSSL::DSA;
+        require Crypt::OpenSSL::DSA::Signature;
     };
 
     # Generate Public Key from XML
@@ -484,8 +477,17 @@ sub _verify_dsa {
 
     # Decode signature and verify
     my $bin_signature = decode_base64($sig);
+
+    # Binary Signature is stored as a concatonation of r and s
+    my ($r, $s) = unpack('a20 a20', $bin_signature);
+
+    # Create a new Signature Object from r and s
+    my $sigobj = Crypt::OpenSSL::DSA::Signature->new();
+    $sigobj->set_r($r);
+    $sigobj->set_s($s);
+
     # DSA signatures are limited to a message body of 20 characters, so a sha1 digest is taken
-    return 1 if ($dsa_pub->verify( $self->{digest_method}->($canonical),  $bin_signature ));
+    return 1 if ($dsa_pub->do_verify( $self->{digest_method}->($canonical),  $sigobj ));
     return 0;
 }
 
@@ -634,12 +636,6 @@ sub _load_x509_key {
     }
 }
 
-#TODO Remove unused?
-sub _set_key_info {
-    my $self = shift;
-
-}
-
 sub _load_cert_file {
     my $self = shift;
 
@@ -790,7 +786,7 @@ sub _canonicalize_xml {
         }
         elsif ($alg eq TRANSFORM_C14N_COMMENTS) {
             print ("        toStringC14N_Comments\n") if $DEBUG;
-            $xml = $xml->toStringC14N(1)
+            $xml = $xml->toStringC14N(1);
         }
         elsif ($alg eq TRANSFORM_C14N_V1_1) {
            print ("        toStringC14N_v1_1\n") if $DEBUG;
@@ -798,7 +794,7 @@ sub _canonicalize_xml {
         }
         elsif ($alg eq TRANSFORM_C14N_V1_1_COMMENTS) {
             print ("        toStringC14N_v1_1_Comments\n") if $DEBUG;
-            $xml = $xml->toStringC14N_v1_1(1)
+            $xml = $xml->toStringC14N_v1_1(1);
         }
         elsif ($alg eq TRANSFORM_EXC_C14N) {
             print ("        toStringEC14N\n") if $DEBUG;
@@ -806,7 +802,7 @@ sub _canonicalize_xml {
         }
         elsif ($alg eq TRANSFORM_EXC_C14N_COMMENTS) {
             print ("        toStringEC14N_Comments\n") if $DEBUG;
-            $xml = $xml->toStringEC14N(1)
+            $xml = $xml->toStringEC14N(1);
         }
         else {
             die "Unsupported transform: $alg";
@@ -826,7 +822,7 @@ XML::Sig
 
 =head1 VERSION
 
-version 0.30
+version 0.31
 
 =head1 SYNOPSIS
 
@@ -952,7 +948,7 @@ Now, let's insert a signature:
 
 =item L<Digest::SHA>
 
-=item L<XML::XPath>
+=item L<XML::LibXML>
 
 =item L<MIME::Base64>
 
@@ -962,6 +958,12 @@ Now, let's insert a signature:
 
 =item L<Crypt::OpenSSL::RSA>
 
+=item L<Crypt::OpenSSL::DSA>
+
+=item L<Crypt::OpenSSL::DSA::Signature>
+
+=item L<Crypt::OpenSSL::X509>
+
 =back
 
 =head1 USAGE
@@ -984,9 +986,17 @@ This module supports the following canonicalization methods and transforms:
 
 =over
 
-=item EXC-X14N#
+=item REC-xml-c14n-20010315#
 
-=item EXC-X14#WithComments
+=item REC-xml-c14n-20010315#WithComments
+
+=item REC-xml-c14n11-20080502
+
+=item REC-xml-c14n11-20080502#WithComments
+
+=item xml-exc-c14n#
+
+=item xml-exc-c14n#WithComments
 
 =item Enveloped Signature
 
@@ -1019,6 +1029,9 @@ When using XML::Sig exclusively to verify a signature, no key needs to be
 specified during initialization given that the public key should be
 transmitted with the signature.
 
+XML::Sig checks all signature in the provided xml and will fail should any
+signature pointing to an existing ID in the XML fail to verify
+
 =item B<signer_cert()>
 
 Following a successful verify with an X509 certificate, returns the
@@ -1047,24 +1060,6 @@ option is used only when generating signatures with the "x509"
 option. This certificate will be embedded in the signed document, and
 should match the private key used for the signature.
 
-=item B<canonicalizer>
-
-The XML canonicalization library to use. Options currently are:
-
-XML::Canonical was removed as an option due to its age
-
-=over
-
-=item XML::CanonicalizerXML (default)
-
-=back
-
-=item B<exclusive>
-
-The XML::CanonicalizerXML exclusive method.  exclusive is an int to specify exclusive canonicalization (1 = exclusive, 0 = non-exclusive, 2 = exclusive v1.1)
-
-default = 1
-
 =item B<x509>
 
 Takes a true (1) or false (0) value and indicates how you want the
@@ -1080,7 +1075,7 @@ L<http://www.w3.org/TR/xmldsig-core/>
 
 =head1 VERSION CONTROL
 
-L<http://github.com/byrnereese/perl-XML-Sig>
+L<https://github.com/perl-net-saml2/perl-XML-Sig>
 
 =head1 AUTHORS and CREDITS
 
@@ -1092,6 +1087,8 @@ signatures.
 
 Net::SAML2 embedded version amended by Chris Andrews <chris@nodnol.org>.
 
+Maintainer: Timothy Legge <timlegge@cpan.org>
+
 =head1 AUTHOR
 
 Original Author: Byrne Reese <byrne@cpan.org>
